@@ -8,23 +8,21 @@ setClass("UCSCSession",
 
 # gets an 'hgsid' to initialize the session
 setMethod("initialize", "UCSCSession",
-          function(.Object, url = "http://genome.ucsc.edu/cgi-bin/",
+          function(.Object, url = "https://genome.ucsc.edu/cgi-bin/",
                    user = NULL, session = NULL, force = FALSE, ...)
           {
             .Object@url <- url
             .Object@views <- new.env()
             gwURL <- ucscURL(.Object, "gateway", force=force)
-            gw <- httpGet(gwURL, cookiefile = tempfile(), header = TRUE,
-                          .parse=FALSE, ...)
-            if (grepl("redirectTd", gw)) {
-                url <- sub(".*?a href=\"h([^[:space:]]+cgi-bin/).*", "h\\1", gw)
-                return(initialize(.Object, url, user=user, session=session,
-                                  force=TRUE, ...))
-            }
-            cookie <- grep("Set-[Cc]ookie: hguid[^=]*=", gw)
-            if (!length(cookie))
+            config <- config(followlocation = 1, nobody = 1,
+                             cookiefile = tempfile(),
+                             useragent = "rtracklayer", ...)
+            ## nobody = 1 seems to be ignored and HEAD() doesn't work here
+            response <- GET(gwURL, config)
+            cookie <- response$headers[["set-cookie"]]
+            if (is.null(cookie))
               stop("Failed to obtain 'hguid' cookie")
-            hguid <- sub(".*Set-Cookie: (hguid[^=]*=[^;]*);.*", "\\1", gw)
+            hguid <- strsplit(cookie, split = ";", fixed = TRUE)[[1L]][[1L]]
             .Object@hguid <- hguid
             if (!is.null(user) && !is.null(session)) { ## bring in other session
               ucscGet(.Object, "tracks",
@@ -199,18 +197,6 @@ setReplaceMethod("range", "UCSCTableQuery",
                    x
                  })
 
-setGeneric("trackName", function(x, ...) standardGeneric("trackName"))
-setMethod("trackName", "UCSCTableQuery", function(x) {
-  .Defunct("tableName", msg = "track is meaningless now you only go by the table")
-})
-
-setGeneric("trackName<-",
-           function(x, ..., value) standardGeneric("trackName<-"))
-setReplaceMethod("trackName", "UCSCTableQuery", function(x, value)
-                 {
-                   .Defunct("tableName<-", msg = "track is meaningless now you only go by the table")
-                 })
-
 setGeneric("tableName", function(x, ...) standardGeneric("tableName"))
 setMethod("tableName", "UCSCTableQuery", function(x) x@table)
 
@@ -261,11 +247,11 @@ normTableQueryRange <- function(range, genome, max.length = 1000L) {
 }
 
 ucscTables <- function(genome, track) {
-  url <- "http://genome.ucsc.edu/cgi-bin/hgTables"
+  url <- "https://genome.ucsc.edu/cgi-bin/hgTables"
   if (!isSingleString(genome))
     stop("'genome' must be a single non-NA string")
   # check genome is valid or not
-  doc <- httpGet(url, c(db = genome))
+  doc <- rtracklayerGET(url, query = list(db=genome))
   genomes <- unlist(getNodeSet(doc, "//select[@name='db']/option/@value"))
   if (!(genome %in% unname(genomes)))
     stop("Invalid genome :'", genome, "'")
@@ -274,7 +260,7 @@ ucscTables <- function(genome, track) {
   track <- normArgTrack(track, trackids)
   # retrieve tables for a track
   form <- c(db = genome, hgta_group = "allTracks", hgta_track = track)
-  doc <- httpGet(url, form)
+  doc <- rtracklayerGET(url, query = form)
   tables <- unlist(getNodeSet(doc, "//select[@name='hgta_table']/option/@value"))
   unname(tables)
 }
@@ -288,19 +274,16 @@ setMethod("ucscTableQuery", "UCSCSession",
 normArgTrack <- function(name, trackids) {
   if (!isSingleString(name))
     stop("'track' must be a single string")
-  if (!(name %in% trackids)) {
-    mapped_name <- trackids[name]
-    if (is.na(mapped_name))
+  if (name %in% names(trackids))
+      unname(trackids[name])
+  else
       stop("Unknown track: ", name)
-    name <- mapped_name
-  }
-  unname(name)
 }
 
 setMethod("ucscTableQuery", "character",
           function(x, track = NULL, range =  NULL, table = NULL,
                    names = NULL, intersectTrack = NULL, check = TRUE, hubUrl = NULL,
-                   genome = NULL, url = "http://genome.ucsc.edu/cgi-bin/") {
+                   genome = NULL, url = "https://genome.ucsc.edu/cgi-bin/") {
               stopifnot(isSingleString(x))
               if (!is.null(intersectTrack))
                 stop("intersectTrack is no longer supported")
@@ -360,7 +343,8 @@ setReplaceMethod("hubUrl", "UCSCTableQuery", function(x, value) {
 
 ## gets the track names available from the table browser
 ucscTableTracks <- function(genome) {
-  doc <- httpGet("http://genome-euro.ucsc.edu/cgi-bin/hgTables", c(db = genome, hgta_group = "allTracks"))
+  url <- "https://genome.ucsc.edu/cgi-bin/hgTables"
+  doc <- rtracklayerGET(url, query = list(db=genome, hgta_group="allTracks"))
   label_path <- "//select[@name = 'hgta_track']/option/text()"
   labels <- sub("\n.*$", "", sapply(getNodeSet(doc, label_path), xmlValue))
   track_path <- "//select[@name = 'hgta_track']/option/@value"
@@ -450,7 +434,7 @@ setMethod("show", "UCSCSchema", function(object) {
 setMethod("track", "UCSCSession",
           function(object, name, ...)
           {
-            .Defunct(msg = "track is meaningless now you only go by the table")
+              track(ucscTableQuery(object, table=name, ...))
           })
 
 ## download a trackSet by name
@@ -1292,7 +1276,8 @@ setMethod("export", c("UCSCData", "UCSCFile"),
                               args[!lineArgs], trackLine = trackLine))
             release(m, con)
             if (index)
-              indexTrack(FileForFormat(resource(file), subformat), skip = 1L)
+              indexTrack(BiocIO::FileForFormat(resource(file), subformat),
+                         skip = 1L)
             else invisible(file)
           })
 
@@ -1379,33 +1364,6 @@ setMethod("import", "UCSCFile",
             GRangesList(tsets, compress=FALSE)
           })
 
-
-
-setMethod("login", "UCSCSession", function(x, username, password) {
-  ucscPost(x, "hgLogin", list(hgLogin.do.displayLogin = "Login",
-                              hgLogin_userName = username,
-                              hgLogin_password = password))
-  
-})
-
-setMethod("saveView", "UCSCView", function(x, name, username, password,
-                                           share = TRUE)
-          {
-            if (!missing(username))
-              login(browserSession(x), username, password)
-            ucscPost(x, "hgSession",
-                     list(hgS_newSessionName = name,
-                          hgS_newSessionShare = if (share) "on" else "off"))
-          })
-
-setMethod("restoreView", "UCSCSession",
-          function(x, name, username, password) {
-            if (!missing(username))
-              login(x, username, password)
-            ucscPost(x, "hgSession",
-                     setNames(list("use"), paste0("hgS_load_", name)))
-          })
-
 ############ INTERNAL API ############
 
 ## every cgi variable is stored in the 'cart'
@@ -1424,6 +1382,7 @@ setMethod("ucscCart", "UCSCSession",
             mat <- matrix(unlist(pairs), nrow = 2)
             vals <- mat[2,]
             names(vals) <- mat[1,]
+            vals[["db"]] <- sub(".*_", "", vals[["db"]])
             new("ucscCart", vals)
           })
 setMethod("ucscCart", "UCSCView",
@@ -1452,7 +1411,7 @@ setClass("ucscTracks",
 setGeneric("ucscTracks", function(object, ...) standardGeneric("ucscTracks"))
 
 setMethod("ucscTracks", "UCSCSession",
-          function(object, form = list())
+          function(object, form = list(pix=800))
           {
             tracks <- ucscGet(object, "tracks", form)
             nodes <- getNodeSet(tracks, "//select/option[@selected]/text()")
@@ -1460,7 +1419,7 @@ setMethod("ucscTracks", "UCSCSession",
             nodes <- getNodeSet(tracks, "//select/@name")
             trackIds <- unlist(nodes)
             ##trackIds <- sapply(nodes, xmlValue)
-            nodes <- getNodeSet(tracks, "//select/../a[not(@class)]/text()")
+            nodes <- getNodeSet(tracks, "//select/../a[not(@class='pennantIconText')]/text()")
             nms <- sapply(nodes, xmlValue)
             names(trackIds) <- sub("^ ", "", nms)
             new("ucscTracks", ids = trackIds, modes = trackModes)
@@ -1484,7 +1443,7 @@ setMethod("ucscTrackModes", "ucscTracks",
 ucscGenomes <- function(organism=FALSE) {
   stopifnot(isTRUEorFALSE(organism))
   names <- c("db", "species", "date", "name", "organism")
-  url <- RestUri("http://api.genome.ucsc.edu/")
+  url <- RestUri("https://api.genome.ucsc.edu/")
   response <- read(url$list$ucscGenomes)
   genomes <- response[[5]]
   genomeNames <- names(genomes)
@@ -1556,8 +1515,8 @@ setMethod("ucscForm", "UCSCView",
               list(hgsid = as.character(object@hgsid))
             else list()
           })
-setOldClass("FileUploadInfo")
-setMethod("ucscForm", "FileUploadInfo",
+setOldClass("form_file")
+setMethod("ucscForm", "form_file",
           function(object, genome = NA_character_, ...)
           {
             form <- list(Submit = "Submit", hgt.customFile = object)
@@ -1568,17 +1527,16 @@ setMethod("ucscForm", "FileUploadInfo",
 setMethod("ucscForm", "SimpleGRangesList",
           function(object, format, ...)
           {
-            lines <- export(object, format = "ucsc", subformat = format, ...)
-            text <- paste(paste(lines, collapse = "\n"), "\n", sep = "")
-            filename <- paste("track", format, sep = ".")
-            upload <- fileUpload(filename, text, "text/plain")
+            path <- tempfile()
+            export(object, path, format = "ucsc", subformat = format, ...)
+            upload <- upload_file(path, type = "text/plain")
             genome <- singleGenome(genome(object))
             ucscForm(upload, genome)
           })
 setMethod("ucscForm", "BiocFile",
           function(object, genome, ...)
           {
-            upload <- fileUpload(path(object), "text/plain")
+            upload <- upload_file(path(object), type = "text/plain")
             ucscForm(upload, genome)
           })
 
@@ -1615,7 +1573,11 @@ ucscURL <-
 # convenience wrappers for _initialized_ sessions
 ucscShow <- function(object, key, .form = list(), ...)
   httpShow(ucscURL(object, key), .form, ...)
-ucscPost <- function(object, key, .form = list(), ...)
-  httpPost(ucscURL(object, key), .form, ..., cookie = ucscCookie(object))
-ucscGet <- function(object, key, .form = list(), ...)
-  httpGet(ucscURL(object, key), .form, ..., cookie = ucscCookie(object))
+ucscPost <- function(object, key, .form = list()) {
+  cookies <- set_cookies(hguid=gsub("^hguid=", "", ucscCookie(object)))
+  rtracklayerPOST(ucscURL(object, key), cookies, body = .form)
+}
+ucscGet <- function(object, key, .form = list()) {
+  cookies <- set_cookies(hguid=gsub("^hguid=", "", ucscCookie(object)))
+  rtracklayerGET(ucscURL(object, key), cookies, query = .form)
+}
